#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "channel.h"
#include "channel_mode.h"
#include "ircd.h"
#include "hash.h"
#include "numeric.h"
#include "s_conf.h"
#include "s_serv.h"
#include "s_user.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "vchannel.h"

static void ms_svstopic(struct Client *, struct Client *, int, char **);

struct Message svstopic_msgtab = {
	"SVSTOPIC", 0, 0, 5, 0, MFLG_SLOW, 0,
	{m_unregistered, m_ignore, ms_svstopic, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&svstopic_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&svstopic_msgtab);
}

const char *_version = "$Revision: 1.0 $";
#endif

/*
 * ms_svstopic
 *      parv[0] = sender prefix
 *      parv[1] = channel name
 *      parv[2] = topic_info
 *      parv[3] = topic_info time
 *      parv[4] = new channel topic
 *
 * Let services always set a topic
 */
static void
ms_svstopic(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct Channel *chptr = NULL;

	if(parc < 5)
		return;

	if(parv[1] && IsChannelName(parv[1])) {
		if((chptr = hash_find_channel(parv[1])) == NULL)
			return;

		if((chptr->topic != NULL) && !strcmp(chptr->topic, parv[4]))
			return;

		set_channel_topic(chptr, parv[4], parv[2], atoi(parv[3]));

		sendto_server(source_p, NULL, chptr, CAP_UID, NOCAPS, NOFLAGS,
		    ":%s SVSTOPIC %s %s %ld :%s",
		    ID(source_p), chptr->chname,
		    chptr->topic_info == NULL ? "" : chptr->topic_info,
		    (unsigned long)chptr->topic_time, chptr->topic == NULL ? "" : chptr->topic);

		sendto_server(source_p, NULL, chptr, NOCAPS, CAP_UID, NOFLAGS,
		    ":%s SVSTOPIC %s %s %ld :%s",
		    source_p->name, chptr->chname,
		    chptr->topic_info == NULL ? "" : chptr->topic_info,
		    (unsigned long)chptr->topic_time, chptr->topic == NULL ? "" : chptr->topic);

		sendto_channel_local(ALL_MEMBERS,
		    chptr, ":%s TOPIC %s :%s",
		    ConfigServerHide.hide_servers ? me.name : source_p->name,
		    chptr->chname, chptr->topic == NULL ? "" : chptr->topic);

	}
}
